/*
==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--

Dia-Semver

Copyright (C) 2018-2022  Anonymous

There are several releases over multiple years,
they are listed as ranges, such as: "2018-2022".

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

::--::--::--::--::--::--::--::--::--::--::--::--::--::--::--::--
*/

//! # Error

use alloc::{
    borrow::Cow,
    fmt::{self, Display, Formatter},
};

#[cfg(feature="std")]
use {
    alloc::string::ToString,
    std::io,
};

/// # Error
#[derive(Debug)]
pub struct Error {
    line: u32,
    module_path: &'static str,
    msg: Option<Cow<'static, str>>,
}

impl Error {

    /// # Makes new instance
    pub (crate) const fn new(line: u32, module_path: &'static str, msg: Option<Cow<'static, str>>) -> Self {
        Self {
            line,
            module_path,
            msg,
        }
    }

    /// # Line
    pub const fn line(&self) -> u32 {
        self.line
    }

    /// # Module path
    pub const fn module_path(&self) -> &str {
        self.module_path
    }

    /// # Error message
    pub fn msg(&self) -> Option<&str> {
        self.msg.as_deref()
    }

}

impl Display for Error {

    fn fmt(&self, f: &mut Formatter) -> Result<(), fmt::Error> {
        match self.msg.as_ref() {
            Some(msg) => write!(
                f, "[{tag}][{module_path}-{line}] {msg}", tag=crate::TAG, line=self.line, module_path=self.module_path, msg=msg,
            ),
            None => write!(f, "[{tag}][{module_path}-{line}]", tag=crate::TAG, line=self.line, module_path=self.module_path),
        }
    }

}

#[cfg(feature="std")]
impl From<Error> for io::Error {

    fn from(err: Error) -> Self {
        io::Error::new(io::ErrorKind::Other, err.to_string())
    }

}
