/*
==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--

Dia-Semver

Copyright (C) 2018-2022  Anonymous

There are several releases over multiple years,
they are listed as ranges, such as: "2018-2022".

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

::--::--::--::--::--::--::--::--::--::--::--::--::--::--::--::--
*/

//! # Parse error messages
//!
//! These are error messages while parsing [`Semver`][struct:Semver].
//!
//! [struct:Semver]: ../struct.Semver.html

macro_rules! err_prefix { () => { "parser:" }}

/// # Empty string
pub const EMPTY: &str = concat!(err_prefix!(), "empty");

/// # Invalid major
pub const INVALID_MAJOR: &str = concat!(err_prefix!(), "invalid-major");

/// # Invalid minor
pub const INVALID_MINOR: &str = concat!(err_prefix!(), "invalid-minor");

/// # Missing minor
pub const MISSING_MINOR: &str = concat!(err_prefix!(), "missing-minor");

/// # Invalid patch
pub const INVALID_PATCH: &str = concat!(err_prefix!(), "invalid-patch");

/// # Missing patch
pub const MISSING_PATCH: &str = concat!(err_prefix!(), "missing-patch");

/// # Invalid token
pub const INVALID_TOKEN: &str = concat!(err_prefix!(), "invalid-token");

/// # Invalid pre-release
pub const INVALID_PRE_RELEASE: &str = concat!(err_prefix!(), "invalid-pre-release");

/// # Invalid build metadata
pub const INVALID_BUILD_METADATA: &str = concat!(err_prefix!(), "invalid-build-metadata");

/// # Too large
pub const TOO_LARGE: &str = concat!(err_prefix!(), "too-large");

/// # Parser's internal error
pub const PARSER_ERROR: &str = concat!(err_prefix!(), "parser-error");
